==> pan-0.102.tar.gz: pan-0.102/pan/tasks/queue.h <==
/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _Queue_h_
#define _Queue_h_

#include <map>
#include <set>
#include <vector>
#include <pan/general/quark.h>
#include <pan/tasks/nntp-pool.h>
#include <pan/tasks/socket.h>
#include <pan/tasks/adaptable-set.h>
#include <pan/tasks/task.h>
#include <pan/tasks/task-weak-ordering.h>

namespace pan
{
  class NNTP;
  class ServerInfo;

  typedef AdaptableSet<Task*, TaskWeakOrdering> TaskSet;

  /**
   * A Queue helper that saves tasks to disk and restores them from disk.
   *
   * @ingroup tasks
   * @see NZB
   */
  struct TaskArchive {
    virtual ~TaskArchive () { }
    virtual void save_tasks (const std::vector<Task*>& saveme) { }
    virtual void load_tasks (std::vector<Task*>& setme) { }
  };

  /**
   * Queues up tasks to be processed by NNTP objects as news server
   * connections become available.
   *
   * @ingroup tasks
   */
  class Queue:
    protected TaskSet,
    public NNTP::Source,
    private NNTP_Pool::Listener
  {
    public:
      Queue (const ServerInfo&, TaskArchive&, Socket::Creator*, int idle_update_interval_secs=15);
      virtual ~Queue ();

      typedef std::vector<Task*> task_list_t;
      void stop_tasks    (const task_list_t&);
      void restart_tasks (const task_list_t&);
      void remove_tasks  (const task_list_t&);
      void move_up       (const task_list_t&);
      void move_down     (const task_list_t&);
      void move_top      (const task_list_t&);
      void move_bottom   (const task_list_t&);
      void add_tasks     (const task_list_t&);

      void set_online (bool online);
      void add_task (Task*);
      void remove_task (Task*);
      void remove_latest_task ();
      bool is_online () const { return _is_online; }
      void upkeep ();

      double get_speed_KiBps () const;
      void get_task_speed_KiBps (const Task*, double& setme_KiBps, int& setme_connections) const;
      void get_connection_counts (int& setme_active, int& setme_idle, int& setme_connecting) const;
      struct ServerConnectionCounts {
        Quark server_id;
        int active, idle, connecting;
        double KiBps;
        ServerConnectionCounts(): active(0), idle(0), connecting(0), KiBps(0.0) {}
      };
      void get_full_connection_counts (std::vector<ServerConnectionCounts>& setme) const;

    public:
      enum TaskState { RUNNING, QUEUED, STOPPED, REMOVING };

      /**
       * An ordered collection of tasks and their corresponding TaskState s.
       */ 
      struct tasks_t {
        task_list_t tasks;
        typedef std::map<Task*,TaskState> states_t;
        states_t states;
        TaskState get_state (Task* task) const {
          states_t::const_iterator it = states.find (task);
          return it == states.end() ? STOPPED : it->second;
        }
      };
      void get_tasks (tasks_t& setme);

    public:

      /** Interface class for objects that listen to a Queue's events */
      struct Listener {
        virtual ~Listener () {}
        virtual void on_queue_task_active_changed (Queue&, Task&, bool active) = 0;
        virtual void on_queue_tasks_added (Queue&, task_list_t&, int index) = 0;
        virtual void on_queue_task_removed (Queue&, Task&, int index) = 0;
        virtual void on_queue_task_moved (Queue&, Task&, int new_index, int old_index) = 0;
        virtual void on_queue_connection_count_changed (Queue&, int count) = 0;
        virtual void on_queue_size_changed (Queue&, int active, int total) = 0;
        virtual void on_queue_online_changed (Queue&, bool online) = 0;
      };
      void add_listener (Listener *);
      void remove_listener (Listener *);

    public: // inherited from NNTP::Source
      virtual void check_in (NNTP*, bool is_ok);

    private: // inherited from NNTP_Pool::Listener
      virtual void on_nntp_available (const Quark& server);

    protected:
      void process_task (Task *);
      const ServerInfo& _server_info;
      bool _is_online;
      Task* find_first_task_needing_server (const Quark& server);
      bool find_best_server (const quarks_t& quarks, Quark& setme);
      bool task_is_active (const Task*) const;

      typedef std::map<NNTP*,Task*> nntp_to_task_t;
      nntp_to_task_t _nntp_to_task;

      std::set<Task*> _removing;
      std::set<Task*> _stopped;
      Socket::Creator * _socket_creator;

    protected:
      virtual void fire_items_added   (task_list_t&, int index);
      virtual void fire_item_removed (Task*&, int index);
      virtual void fire_item_moved   (Task*&, int index, int old_index);

    private:
      typedef std::set<Listener*> listeners_t;
      typedef listeners_t::iterator lit;
      listeners_t _listeners;
      void fire_task_active_changed (Task*, bool);
      void fire_connection_count_changed (int count);
      void fire_size_changed (int active, int total);
      void fire_online_changed (bool online);

    private:
      typedef std::map<Quark,NNTP_Pool*> pools_t;
      pools_t _pools;
      NNTP_Pool& get_pool (const Quark& server);

    private:
      TaskArchive& _archive;
  };
}

#endif

==> pan-0.103.tar.gz: pan-0.103/pan/tasks/queue.h <==
/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _Queue_h_
#define _Queue_h_

#include <map>
#include <set>
#include <vector>
#include <pan/general/quark.h>
#include <pan/tasks/nntp-pool.h>
#include <pan/tasks/socket.h>
#include <pan/tasks/adaptable-set.h>
#include <pan/tasks/task.h>
#include <pan/tasks/task-weak-ordering.h>

namespace pan
{
  class NNTP;
  class ServerInfo;

  typedef AdaptableSet<Task*, TaskWeakOrdering> TaskSet;

  /**
   * A Queue helper that saves tasks to disk and restores them from disk.
   *
   * @ingroup tasks
   * @see NZB
   */
  struct TaskArchive {
    virtual ~TaskArchive () { }
    virtual void save_tasks (const std::vector<Task*>& saveme) { }
    virtual void load_tasks (std::vector<Task*>& setme) { }
  };

  /**
   * Queues up tasks to be processed by NNTP objects as news server
   * connections become available.
   *
   * @ingroup tasks
   */
  class Queue:
    protected TaskSet,
    public NNTP::Source,
    private NNTP_Pool::Listener
  {
    public:
      Queue (const ServerInfo&, TaskArchive&, Socket::Creator*, int idle_update_interval_secs=15);
      virtual ~Queue ();

      typedef std::vector<Task*> task_list_t;
      void stop_tasks    (const task_list_t&);
      void restart_tasks (const task_list_t&);
      void remove_tasks  (const task_list_t&);
      void move_up       (const task_list_t&);
      void move_down     (const task_list_t&);
      void move_top      (const task_list_t&);
      void move_bottom   (const task_list_t&);

      enum AddMode { AGE, TOP, BOTTOM };
      void add_tasks     (const task_list_t&, AddMode=AGE);

      void set_online (bool online);
      void add_task (Task*, AddMode=AGE);
      void remove_task (Task*);
      void remove_latest_task ();
      bool is_online () const { return _is_online; }
      void upkeep ();

      double get_speed_KiBps () const;
      void get_task_speed_KiBps (const Task*, double& setme_KiBps, int& setme_connections) const;
      void get_connection_counts (int& setme_active, int& setme_idle, int& setme_connecting) const;
      struct ServerConnectionCounts {
        Quark server_id;
        int active, idle, connecting;
        double KiBps;
        ServerConnectionCounts(): active(0), idle(0), connecting(0), KiBps(0.0) {}
      };
      void get_full_connection_counts (std::vector<ServerConnectionCounts>& setme) const;

    public:
      enum TaskState { RUNNING, QUEUED, STOPPED, REMOVING };

      /**
       * An ordered collection of tasks and their corresponding TaskState s.
       */ 
      struct tasks_t {
        task_list_t tasks;
        typedef std::map<Task*,TaskState> states_t;
        states_t states;
        TaskState get_state (Task* task) const {
          states_t::const_iterator it = states.find (task);
          return it == states.end() ? STOPPED : it->second;
        }
      };
      void get_tasks (tasks_t& setme);

    public:

      /** Interface class for objects that listen to a Queue's events */
      struct Listener {
        virtual ~Listener () {}
        virtual void on_queue_task_active_changed (Queue&, Task&, bool active) = 0;
        virtual void on_queue_tasks_added (Queue&, task_list_t&, int index) = 0;
        virtual void on_queue_task_removed (Queue&, Task&, int index) = 0;
        virtual void on_queue_task_moved (Queue&, Task&, int new_index, int old_index) = 0;
        virtual void on_queue_connection_count_changed (Queue&, int count) = 0;
        virtual void on_queue_size_changed (Queue&, int active, int total) = 0;
        virtual void on_queue_online_changed (Queue&, bool online) = 0;
      };
      void add_listener (Listener *);
      void remove_listener (Listener *);

    public: // inherited from NNTP::Source
      virtual void check_in (NNTP*, bool is_ok);

    private: // inherited from NNTP_Pool::Listener
      virtual void on_nntp_available (const Quark& server);

    protected:
      void process_task (Task *);
      const ServerInfo& _server_info;
      bool _is_online;
      Task* find_first_task_needing_server (const Quark& server);
      bool find_best_server (const quarks_t& quarks, Quark& setme);
      bool task_is_active (const Task*) const;

      typedef std::map<NNTP*,Task*> nntp_to_task_t;
      nntp_to_task_t _nntp_to_task;

      std::set<Task*> _removing;
      std::set<Task*> _stopped;
      Socket::Creator * _socket_creator;

    protected:
      virtual void fire_items_added   (task_list_t&, int index);
      virtual void fire_item_removed (Task*&, int index);
      virtual void fire_item_moved   (Task*&, int index, int old_index);

    private:
      typedef std::set<Listener*> listeners_t;
      typedef listeners_t::iterator lit;
      listeners_t _listeners;
      void fire_task_active_changed (Task*, bool);
      void fire_connection_count_changed (int count);
      void fire_size_changed (int active, int total);
      void fire_online_changed (bool online);

    private:
      typedef std::map<Quark,NNTP_Pool*> pools_t;
      pools_t _pools;
      NNTP_Pool& get_pool (const Quark& server);

    private:
      TaskArchive& _archive;
  };
}

#endif
